searchData={"items":[{"type":"module","title":"EctoInterface","doc":"EctoInterface is a suite of commonly defined APIs for Ecto-based models. Almost all of these came from the normal\ngenerated content in a new phoenix project, simply unifying and expanding on that suite.","ref":"EctoInterface.html"},{"type":"macro","title":"EctoInterface.__using__/1","doc":"Using this expression in your context module:\n\n    use(EctoInterface, [Core.Commerce.Product, :products, :product])\n\nWill automatically define a whole suite of functions for that schema.\n\nYou can also get specific about which changesets are used (default is schema.changeset/1):\n\n    use(EctoInterface, [Core.Commerce.Product, :products, :product, :simple_changeset])\n\nAnd also if you need to differentiate between insert and update changesets:\n\n    use(EctoInterface, [Core.Commerce.Product, :products, :product, :create_changeset, :update_changeset])","ref":"EctoInterface.html#__using__/1"},{"type":"behaviour","title":"EctoInterface.Paginator","doc":"Defines a paginator.\n\nThis module adds a `paginate/3` function to your `Ecto.Repo` so that you can\npaginate through results using opaque cursors.","ref":"EctoInterface.Paginator.html"},{"type":"behaviour","title":"Usage - EctoInterface.Paginator","doc":"defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app\n      use EctoInterface.Paginator\n    end","ref":"EctoInterface.Paginator.html#module-usage"},{"type":"behaviour","title":"Options - EctoInterface.Paginator","doc":"`EctoInterface.Paginator` can take any options accepted by `paginate/3`. This is useful when\nyou want to enforce some options globally across your project.\n\n#","ref":"EctoInterface.Paginator.html#module-options"},{"type":"behaviour","title":"Example - EctoInterface.Paginator","doc":"defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app\n      use EctoInterface.Paginator,\n        limit: 10,                           # sets the default limit to 10\n        maximum_limit: 100,                  # sets the maximum limit to 100\n        include_total_count: true,           # include total count by default\n        total_count_primary_key_field: :uuid # sets the total_count_primary_key_field to uuid for calculate total_count\n    end\n\nNote that these values can be still be overridden when `paginate/3` is called.\n\n#","ref":"EctoInterface.Paginator.html#module-example"},{"type":"behaviour","title":"Use without macros - EctoInterface.Paginator","doc":"If you wish to avoid use of macros or you wish to use a different name for\nthe pagination function you can define your own function like so:\n\n    defmodule MyApp.Repo do\n      use Ecto.Repo, otp_app: :my_app\n\n      def my_paginate_function(queryable, opts \\ [], repo_opts \\ []) do\n        defaults = [limit: 10] # Default options of your choice here\n        opts = Keyword.merge(defaults, opts)\n        EctoInterface.Paginator.paginate(queryable, opts, __MODULE__, repo_opts)\n      end\n    end","ref":"EctoInterface.Paginator.html#module-use-without-macros"},{"type":"function","title":"EctoInterface.Paginator.cursor_for_record/3","doc":"Generate a cursor for the supplied record, in the same manner as the\n`before` and `after` cursors generated by `paginate/3`.\n\nFor the cursor to be compatible with `paginate/3`, `cursor_fields`\nmust have the same value as the `cursor_fields` option passed to it.\n\n#","ref":"EctoInterface.Paginator.html#cursor_for_record/3"},{"type":"function","title":"Example - EctoInterface.Paginator.cursor_for_record/3","doc":"iex> EctoInterface.Paginator.cursor_for_record(%Customer{id: 1}, [:id])\n    \"g3QAAAABZAACaWRhAQ==\"\n\n    iex> EctoInterface.Paginator.cursor_for_record(%Customer{id: 1, name: \"Alice\"}, [id: :asc, name: :desc])\n    \"g3QAAAACZAACaWRhAWQABG5hbWVtAAAABUFsaWNl\"","ref":"EctoInterface.Paginator.html#cursor_for_record/3-example"},{"type":"function","title":"EctoInterface.Paginator.default_fetch_cursor_value/2","doc":"Default function used to get the value of a cursor field from the supplied\nmap. This function can be overridden in the `EctoInterface.Paginator.Config` using the\n`fetch_cursor_value_fun` key.\n\nWhen using named bindings to sort on joined columns it will attempt to get\nthe value of joined column by using the named binding as the name of the\nrelationship on the original Ecto.Schema.\n\n#","ref":"EctoInterface.Paginator.html#default_fetch_cursor_value/2"},{"type":"function","title":"Example - EctoInterface.Paginator.default_fetch_cursor_value/2","doc":"iex> EctoInterface.Paginator.default_fetch_cursor_value(%Customer{id: 1}, :id)\n    1\n\n    iex> EctoInterface.Paginator.default_fetch_cursor_value(%Customer{id: 1, address: %Address{city: \"London\"}}, {:address, :city})\n    \"London\"","ref":"EctoInterface.Paginator.html#default_fetch_cursor_value/2-example"},{"type":"callback","title":"EctoInterface.Paginator.paginate/3","doc":"Fetches all the results matching the query within the cursors.","ref":"EctoInterface.Paginator.html#c:paginate/3"},{"type":"callback","title":"Options - EctoInterface.Paginator.paginate/3","doc":"* `:after` - Fetch the records after this cursor.\n  * `:before` - Fetch the records before this cursor.\n  * `:cursor_fields` - The fields with sorting direction used to determine the\n  cursor. In most cases, this should be the same fields as the ones used for sorting in the query.\n  When you use named bindings in your query they can also be provided.\n  * `:fetch_cursor_value_fun` function of arity 2 to lookup cursor values on returned records.\n  Defaults to `EctoInterface.Paginator.default_fetch_cursor_value/2`\n  * `:include_total_count` - Set this to true to return the total number of\n  records matching the query. Note that this number will be capped by\n  `:total_count_limit`. Defaults to `false`.\n  * `:total_count_primary_key_field` - Running count queries on specified column of the table\n  * `:limit` - Limits the number of records returned per page. Note that this\n  number will be capped by `:maximum_limit`. Defaults to `50`.\n  * `:maximum_limit` - Sets a maximum cap for `:limit`. This option can be useful when `:limit`\n  is set dynamically (e.g from a URL param set by a user) but you still want to\n  enfore a maximum. Defaults to `500`.\n  * `:sort_direction` - The direction used for sorting. Defaults to `:asc`.\n  It is preferred to set the sorting direction per field in `:cursor_fields`.\n  * `:total_count_limit` - Running count queries on tables with a large number\n  of records is expensive so it is capped by default. Can be set to `:infinity`\n  in order to count all the records. Defaults to `10,000`.","ref":"EctoInterface.Paginator.html#c:paginate/3-options"},{"type":"callback","title":"Repo options - EctoInterface.Paginator.paginate/3","doc":"This will be passed directly to `Ecto.Repo.all/2`, as such any option supported\nby this function can be used here.","ref":"EctoInterface.Paginator.html#c:paginate/3-repo-options"},{"type":"callback","title":"Simple example - EctoInterface.Paginator.paginate/3","doc":"query = from(p in Post, order_by: [asc: p.inserted_at, asc: p.id], select: p)\n\n    Repo.paginate(query, cursor_fields: [:inserted_at, :id], limit: 50)","ref":"EctoInterface.Paginator.html#c:paginate/3-simple-example"},{"type":"callback","title":"Example with using custom sort directions per field - EctoInterface.Paginator.paginate/3","doc":"query = from(p in Post, order_by: [asc: p.inserted_at, desc: p.id], select: p)\n\n    Repo.paginate(query, cursor_fields: [inserted_at: :asc, id: :desc], limit: 50)","ref":"EctoInterface.Paginator.html#c:paginate/3-example-with-using-custom-sort-directions-per-field"},{"type":"callback","title":"Example with sorting on columns in joined tables - EctoInterface.Paginator.paginate/3","doc":"from(\n      p in Post,\n      as: :posts,\n      join: a in assoc(p, :author),\n      as: :author,\n      preload: [author: a],\n      select: p,\n      order_by: [\n        {:asc, a.name},\n        {:asc, p.id}\n      ]\n    )\n\n    Repo.paginate(query, cursor_fields: [{{:author, :name}, :asc}, id: :asc], limit: 50)\n\nWhen sorting on columns in joined tables it is necessary to use named bindings. In\nthis case we name it `author`. In the `cursor_fields` we refer to this named binding\nand its column name.\n\nTo build the cursor EctoInterface.Paginator uses the returned Ecto.Schema. When using a joined\ncolumn the returned Ecto.Schema won't have the value of the joined column\nunless we preload it. E.g. in this case the cursor will be build up from\n`post.id` and `post.author.name`. This presupposes that the named of the\nbinding is the same as the name of the relationship on the original struct.\n\nOne level deep joins are supported out of the box but if we join on a second\nlevel, e.g. `post.author.company.name` a custom function can be supplied to\nhandle the cursor value retrieval. This also applies when the named binding\ndoes not map to the name of the relationship.","ref":"EctoInterface.Paginator.html#c:paginate/3-example-with-sorting-on-columns-in-joined-tables"},{"type":"callback","title":"Example - EctoInterface.Paginator.paginate/3","doc":"from(\n      p in Post,\n      as: :posts,\n      join: a in assoc(p, :author),\n      as: :author,\n      join: c in assoc(a, :company),\n      as: :company,\n      preload: [author: a],\n      select: p,\n      order_by: [\n        {:asc, a.name},\n        {:asc, p.id}\n      ]\n    )\n\n    Repo.paginate(query,\n      cursor_fields: [{{:company, :name}, :asc}, id: :asc],\n      fetch_cursor_value_fun: fn\n        post, {:company, name} ->\n          post.author.company.name\n\n        post, field ->\n          EctoInterface.Paginator.default_fetch_cursor_value(post, field)\n      end,\n      limit: 50\n    )","ref":"EctoInterface.Paginator.html#c:paginate/3-example"},{"type":"module","title":"EctoInterface.Paginator.Config","doc":"The configuration record of a pagination request.","ref":"EctoInterface.Paginator.Config.html"},{"type":"function","title":"EctoInterface.Paginator.Config.new/1","doc":"","ref":"EctoInterface.Paginator.Config.html#new/1"},{"type":"function","title":"EctoInterface.Paginator.Config.validate!/1","doc":"","ref":"EctoInterface.Paginator.Config.html#validate!/1"},{"type":"type","title":"EctoInterface.Paginator.Config.t/0","doc":"","ref":"EctoInterface.Paginator.Config.html#t:t/0"},{"type":"module","title":"EctoInterface.Paginator.Page","doc":"Defines a page.","ref":"EctoInterface.Paginator.Page.html"},{"type":"module","title":"Fields - EctoInterface.Paginator.Page","doc":"* `entries` - a list entries contained in this page.\n* `metadata` - metadata attached to this page.","ref":"EctoInterface.Paginator.Page.html#module-fields"},{"type":"type","title":"EctoInterface.Paginator.Page.t/0","doc":"","ref":"EctoInterface.Paginator.Page.html#t:t/0"},{"type":"module","title":"EctoInterface.Paginator.PageMetadata","doc":"Defines page metadata.","ref":"EctoInterface.Paginator.PageMetadata.html"},{"type":"module","title":"Fields - EctoInterface.Paginator.PageMetadata","doc":"* `after` - an opaque cursor representing the last row of the current page.\n* `before` - an opaque cursor representing the first row of the current page.\n* `limit` - the maximum number of entries that can be contained in this page.\n* `total_count` - the total number of entries matching the query.\n* `total_count_cap_exceeded` - a boolean indicating whether the `:total_count_limit`\nwas exceeded.","ref":"EctoInterface.Paginator.PageMetadata.html#module-fields"},{"type":"type","title":"EctoInterface.Paginator.PageMetadata.opaque_cursor/0","doc":"","ref":"EctoInterface.Paginator.PageMetadata.html#t:opaque_cursor/0"},{"type":"type","title":"EctoInterface.Paginator.PageMetadata.t/0","doc":"","ref":"EctoInterface.Paginator.PageMetadata.html#t:t/0"},{"type":"module","title":"EctoInterface.PubSub","doc":"A group of helper methods for broadcasting/listening to data through a pubsub system.","ref":"EctoInterface.PubSub.html"},{"type":"module","title":"EctoInterface.Read","doc":"All functions that help read data from the database.","ref":"EctoInterface.Read.html"},{"type":"module","title":"EctoInterface.Read.Count","doc":"All functions that help read data from the database.","ref":"EctoInterface.Read.Count.html"},{"type":"module","title":"EctoInterface.Read.Get","doc":"All functions that help read data from the database.","ref":"EctoInterface.Read.Get.html"},{"type":"module","title":"EctoInterface.Read.List","doc":"All functions that help read data from the database.","ref":"EctoInterface.Read.List.html"},{"type":"module","title":"EctoInterface.Read.Random","doc":"All functions that help read data from the database.","ref":"EctoInterface.Read.Random.html"},{"type":"module","title":"EctoInterface.Read.Slug","doc":"If you have Slugy installed you can use these functions to find a record by it's slug","ref":"EctoInterface.Read.Slug.html"},{"type":"exception","title":"EctoInterface.Paginator.Config.ArgumentError","doc":"","ref":"EctoInterface.Paginator.Config.ArgumentError.html"}],"content_type":"text/markdown"}